#include  "string.h"
// Tool-specific include files
#include  "iomaxq200x.h"  // MAXQ2000 registers
#include  "intrinsics.h"  // intrinsic functions

  // Output Buffer
#define buf_len 80
char  buffer[buf_len];
int   ptr;
  // this will append the buffer with text
void  appendbuf(char * text) {
  int i=0;
  while(text[i] && ptr<buf_len)  buffer[ptr++]=text[i++];
}

// Interrupt Service Routines

  // Push-button actions
void  button(int num) {
  long i;
  switch(num)
  {
    // Red Button
    case 0:
      for(i=0;i<100000;) i++; // pause (burn 100,000 cycles)
      break;
    // Other buttons
    case 12: case 11: case 10:
      for(i=0;i<100000;) i++; // pause (burn 100,000 cycles)
      break;
  }
}

  // Ext 10,11,12
#pragma vector=1
__interrupt void isr_module1() {
  int mask;
  // Re-configure interrupts
  mask=__get_interrupt_state(); // save configuration (mask=IMR;)
  __set_interrupt_state(0x95);  // disable mod 1; enable 0,2,4,7
  __reenable_interrupt();       // allow recursive calls (INS=0)

    // Individual ISR
    // Check Int12
    if(EIF1_bit.IE12) {
      EIF1_bit.IE12=0;      // clear int flag
      appendbuf("<Ext12");  // append buffer with text
      button(12);           // push-button action
      appendbuf("Ext12>");  // append buffer with text
    }
    // Check Int11
    if(EIF1_bit.IE11) {
      EIF1_bit.IE11=0;      // clear int flag
      appendbuf("<Ext11");  // append buffer with text
      button(11);           // push-button action
      appendbuf("Ext11>");  // append buffer with text
    }
    // Check Int10
    if(EIF1_bit.IE10) {
      EIF1_bit.IE10=0;      // clear int flag
      appendbuf("<Ext10");  // append buffer with text
      button(10);           // push-button action
      appendbuf("Ext10>");  // append buffer with text
    }

  // Restore interrupt configuration
  IC_bit.INS=1;                 // restore INS=1
  __set_interrupt_state(mask);  // restore mask (IMR=mask;)
}

  // Ext 0
#pragma vector=0
__interrupt void isr_module0() {
  int mask;
  // Re-configure interrupts
  mask=__get_interrupt_state(); // save configuration (mask=IMR;)
  __set_interrupt_state(0x94);  // disable mod 0,1; enable 2,4,7
  __reenable_interrupt();       // allow recursive calls (INS=0)

    // Individual ISR
    EIF0_bit.IE0=0;       // clear int flag
    appendbuf("<Ext0");   // append buffer with text
    button(0);            // Red push-button action
    appendbuf("Ext0>");   // append buffer with text

  // Restore interrupt configuration
  IC_bit.INS=1;                 // restore INS=1
  __set_interrupt_state(mask);  // restore mask (IMR=mask;)
}

  // Timer 1
#pragma vector=4
__interrupt void isr_module4() {
  int mask;
  // Re-configure interrupts
  mask=__get_interrupt_state(); // save configuration (mask=IMR;)
  __set_interrupt_state(0x84);  // disable mod 1,0,4; enable 2,7
  __reenable_interrupt();       // allow recursive calls (INS=0)

    // Individual ISR
    T2CNB1_bit.TF2=0;     // clear interrupt flag
    PO0 |= 0x01;          // set pin hi
    appendbuf("<Timer>"); // append buffer with text
    PO0 &= 0xFE;          // set pin low

  // Restore interrupt configuration
  IC_bit.INS=1;                 // restore INS=1
  __set_interrupt_state(mask);  // restore mask (IMR=mask;)
}

  // UART0 Transmit Done
#pragma vector=2
__interrupt void isr_module2() {
  int mask;
  // Re-configure interrupts
  mask=__get_interrupt_state(); // save configuration (mask=IMR;)
  __set_interrupt_state(0x80);  // disable mod 1,0,4,2; enable 7
  __reenable_interrupt();       // allow recursive calls (INS=0)

    // Individual ISR
    SCON0_bit.TI=0;                     // clear int flag
    if(ptr>0) {
      SBUF0=buffer[0];                  // output next byte
      memmove(buffer,&buffer[1],ptr--); // shift the buffer
    }

  // Restore interrupt configuration
  IC_bit.INS=1;                 // restore INS=1
  __set_interrupt_state(mask);  // restore mask (IMR=mask;)
}

  // Watchdog Timer
#pragma vector=7
__interrupt void isr_module7() {
  WDCN_bit.WDIF=0;          // clear int flag
  __clear_watchdog_timer(); // reset WDT
  SBUF0='.';                // print a symbol via UART0
}



// Application Code
int main(int argc, char* argv[]) {
  long i;
  // Init Port Pin P0.0
  PD0 |= 0x01;      // counfigure direction (output)
  PO0 &= 0xFE;      // set pin P0.0 low
  // Init Interrupts
  WDCN=0x52;        // enable Watchdog int
  // Ext 0,10,11,12
  EIES0=0x01;       // negative edge trigger for int 0
  EIE0 =0x01;       // enable ext 0 itnterrupt
  EIES1=0x1C;       // negative edge trigger for int 10,11,12
  EIE1 =0x1C;       // enable ext 10,11,12 itnterrupts
  // Timer 1
  T2R1=65536-16000; // ovfl every 1ms @ 16MHz
  T2V1=T2R1;        // init counter
  T2CFG1=0x70;      // 16-bit timer mode, clock/128
  T2CNA1=0x88;      // int enabled, start the timer
  // UART0 (serial)
  SCON0=0x40;       // async.mode 1, clock/4
  SMD0=0x06;        // clock/16, int enabled
  PR0=0x3AFB;       // 115200 bps @ 16MHz
  // modules & global
  __set_interrupt_state(0x97);  // enable modules 0,1,2,4,7 (IMR=0x97;)
  __enable_interrupt();         // global enable (IC_bit.IGE=1;)

  ptr=0;
  appendbuf("=Reset=");     // append buffer with text
  while(1) {
    appendbuf("Main");      // append buffer with text
    for(i=0;i<50000;) i++;  // pause (burn 50000 cycles)
  }
}
